/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its lower left-hand corner
   placed at the point (1, 0).
   <pre>{@code
            y                             y

            |                             |
            |                             |       / \
          1 +---------+                 1 +     /     \
            |         |                   |   /         \
            |         |                   |  <           >
            |         |                   |   \         /
            |         |                   |     \     /
            |         |                   |       \ /
    --------+---------+---> x     --------+--------+-------> x
            |         1                   |        1
            |                             |
            |                             |
   }</pre>
<p>
   See the animation <a href="../Example_02.gif">Example_02.gif</a>.
<p>
   <a href="../Example_02.gif"><img src="../Example_02.gif" alt="Example_02.gif"></a>
<p>
   For the i'th frame of the animation, the square's model matrix is
   <pre>{@code
      M = T(1, 0, 0) * Rz(5*i)
        = T(1, 0, 0) * Rz(5*(i-1)) * Rz(5).
   }</pre>
   Notice how new rotations can be accumulated on the right end of
   the matrix product.
<p>
   In the code below, the main loop for creating the animation frames
   looks like this.
   <pre>{@code
      // Translate the square's lower left-hand corner to the point (1,0).
      square_p.matrix = Matrix.translate(1, 0, 0);

      for (int i = 0; i <= 72 ++i)
      {
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_02_Frame%02d.ppm", i));

         square_p.getMatrix().mult( Matrix.rotateZ(5) );
      }
   }</pre>
   As in the previous example, each frame of the animation rotates the
   square another 5 degrees from the previous frame. The difference in
   this example is the single translation of the model before the loop.
   The rotations are being accumulated "after" the translation if you
   read this code from top to bottom (or you read the tranformation
   formula from left-to-right). On the other hand, the rotations are
   being accumulated "before" the translation if you read this code
   from the bottom up (or you read the tranformation formula from
   right-to-left).
<p>
   Here is the scene tree that this program builds for the i'th frame
   of the animation. There is only one Matrix object in the scene. All
   of the transformation information needs to be in that single matrix.
   Notice how the additional transformation needed by each new frame, a
   rotation by 5 degrees, is accumulated on the right of the position's
   matrix.
   <pre>{@code
             Scene
            /     \
           /       \
      Camera        List<Position>
                          |
                          |
                      Position
                      /       \
                     /         \
                Matrix         Square
             T(1,0)*Rz(5i)
   }</pre>
*/
public class Example_02
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      final double near = -1.0;
      scene.getCamera().projOrtho(left, right, bottom, top, near);

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      ModelShading.setColor(axes, Color.red);
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Add a square to the Scene.
      final Model square = new Square("Example_02");
      final Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Translate the square's lower left-hand corner to the point (1,0).
      square_p.setMatrix( Matrix.translate(1, 0, 0) );

      //Rotate the square around its lower left-hand corner.
      for (int i = 0; i < 72; ++i)
      {
         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_02a_Frame%03d.ppm", i));

         // Accumulate rotations of the square in the Position's matrix.
         square_p.getMatrix().mult( Matrix.rotateZ(5) );
       //square_p.setMatrix( square_p.getMatrix().times(Matrix.rotateZ(5)) ); // another way to this
      }


      //Rotate the square around its lower left-hand corner placed at the point (1,0).
      for (int i = 0; i < 72; ++i)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_02b_Frame%03d.ppm", i));

         // Translate the square's lower left-hand corner to the point (1,0,0)
         // and then rotate the square.
         square_p.setMatrix( Matrix.translate(1, 0, 0)
                      .mult( Matrix.rotateZ(5 * i) ) );
       //square_p.setMatrix( Matrix.translate(1, 0, 0)  // another way to this
       //            .times( Matrix.rotateZ(5 * i) ) );
      }
   }
}

/*
Here are the four ways used above to implement T(1,0,0)*Rz(5*i).
Which one is the most efficient?
   square_p.getMatrix().mult(Matrix.rotateZ(5));
   square_p.setMatrix( square_p.getMatrix().times(Matrix.rotateZ(5)) );
   square_p.setMatrix( Matrix.translate(1,0,0).mult(Matrix.rotateZ(5*i)) );
   square_p.setMatrix( Matrix.translate(1,0,0).times(Matrix.rotateZ(5*i)) );
*/
