/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its center, with its center at the origin.
<p>
   Do this by "accumulating" rotations.
<p>
   The last animation showed how the coordinate system gets transformed
   by the code. Notice in that animation that we always "go back" to the
   original coordinate system and then rotate and translate it. This example
   shows we don't need to go back to the original coordinate system. We can
   "save" the rotated coordinate system, translate it, then draw, then
   "return" to the rotated coordinate system, accumulate another rotation
   (on the right), save it again, translate it again, then draw, etc.
   <pre>{@code
      M = Rz(5*i) * T(-0.5, -0.5, 0)
        = Rz(5*(i-1)) * Rz(5) * T(-0.5, -0.5, 0).
          \_________________/
           save this product
   }</pre>
<p>
   This is not a better way to create the animation since it is harder to
   read. But sometimes this idea of "saving" and "restoring" a coordinates
   system can be useful.
*/
public class Example_09c
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      final double near = -1.0;
      scene.getCamera().projOrtho(left, right, bottom, top, near);

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Add a square to the Scene.
      final Model square = new Square("Example_09c");
      final Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Rotate the square around its center, with its center at the origin.
      Matrix save = square_p.getMatrix(); // save the coordinate system
      for (int i = 0; i < 72; ++i)
      {
         // Update the model matrix.
         square_p.setMatrix( save.times( Matrix.translate(-0.5, -0.5, 0) ) );

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_09c_Frame%03d.ppm", i));

          // Accumulate a rotation on the right of the saved matrix.
         save.mult( Matrix.rotateZ(5) );
      }
   }
}
