/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its lower left-hand corner
   placed on the circumference of the unit circle.
<p>
   See the animation <a href="../Example_10.gif">Example_10.gif</a>.
<p>
   <a href="../Example_10.gif"><img src="../Example_10.gif" alt="Example_10.gif"></a>
<p>
   For the i'th frame of the animation, the square's model matrix is
   <pre>{@code
      M = T(cos(5*i), sin(5*i)) * Rz(5*i)
   }</pre>
<p>
   Here is the scene tree that this program builds for the i'th frame
   of the animation. There is only one Matrix object in the scene. All
   of the transformation information needs to be in that single matrix.
   <pre>{@code
             Scene
            /     \
           /       \
      Camera       List<Position>
                         |
                         |
                      Position
                      /      \
                     /        \
                Matrix         Square
            T(c,s)*Rz(5i)
   }</pre>
<p>
   (The code below does show a way to "force" the matrix
   multiplcation to be on the left of the position's matrix.)
*/
public class Example_10c
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      final double near = -1.0;
      scene.getCamera().projOrtho(left, right, bottom, top, near);

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      ModelShading.setColor(axes, Color.red);
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Add a square to the Scene.
      final Model square = new Square("Example_10c");
      final Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Rotate the square around its lower left-hand corner
      // placed on the circumference of the unit circle.
      for (int i = 0; i < 72; ++i)
      {
         // Translate the coordinate system.
         square_p.setMatrix( Matrix.translate(Math.cos(5*i*Math.PI/180),
                                              Math.sin(5*i*Math.PI/180),
                                              0) );
         // Rotate the coordinate system.
         square_p.getMatrix().mult( Matrix.rotateZ(5*i) );

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_10c_Frame%03d.ppm", i));
      }
   }
}
