/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Translate the letter {@link P} along the x-axis while
   rotating around the y-axis.
 <p>
   This is another way to solve this problem.
<p>
   For the i'th frame of the animation, the model matrix is
   <pre>{@code
      M = Tx(0.05*i) * Ry(18*i)
        = Tx(0.05) * Tx(0.05*(i-1)) * Ry(18*(i-1)) * Ry(18)
   }</pre>
   Where {@code Ry} is rotation around the y-axis and {@code Tx} is
   translation in the x-direction. Notice how new rotations can be
   accumulated on the right end of the matrix product and new
   translations can be accumulated on the left end of the product.
<p>
   Using the {@link Matrix#multLeft} method from the {@link Matrix}
   class we accumulate the translations on the left of the model matrix
   in the position object while we use the {@link Matrix#mult} method
   to accumulate rotations on the right of the model matrix.
*/
public class Example_23c
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Set up a perspective camera.
      final double right = 5.0;
      final double left = -1.0;
      final double top = 2.0;
      final double bottom = -top;
      final double near = 1.0;
      scene.getCamera().projPerspective(left, right, bottom, top, near);

      // Create a FrameBuffer to render our scene into.
      int width  = 600;
      int height = 400;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(left, right, bottom, top, 12, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);
      // Push the axes away from where the camera is.
      axes_p.setMatrix( Matrix.translate(0, 0, -near) );

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      ModelShading.setColor(modelP, Color.blue);
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);
      // Push the model away from where the camera is.
      modelP_p.setMatrix( Matrix.translate(0, 0, -near) );

      // Translate along the x-axis while rotating around the y-axis.
      // (This is harder. In fact, this code doesnt work, since it
      // assumes that these rotations and translations commute.)
      for (int i = 0; i <= 100; ++i)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_23a_Frame%03d.ppm", i));

         // Rotate around the y-axis and translate down the x-axis.
         modelP_p.getMatrix().mult( Matrix.rotateY(18) );           // Accumulate a rotation on the right.
         modelP_p.getMatrix().multLeft( Matrix.translate(0.05, 0, 0) ); // Accumulate a translation on the left.
      }
   }
}
