/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Translate the letter {@link P} along the y-axis while
   rotating around the x-axis.
<p>
   See the animation <a href="../Example_24.gif">Example_24.gif</a>.
<p>
   <a href="../Example_24.gif"><img src="../Example_24.gif" alt="Example_24.gif"></a>
<p>
   For the i'th frame of the animation, the model matrix is
   <pre>{@code
      M = Ty(0.05*i) * Rx(18*i)
   }</pre>
   Where {@code Rx} is rotation around the x-axis and {@code Ty} is
   translation in the y-direction. This solution does not try to
   accumulate matrix products. For each frame of the animation, this
   solution builds the model matrix as the product of a translation
   and a rotation.
*/
public class Example_24
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Set up a perspective camera.
      final double right = 2.0;
      final double left = -right;
      final double top = 5.0;
      final double bottom = -1.0;
      final double near = 1.0;
      scene.getCamera().projPerspective(left, right, bottom, top, near);

      // Create a FrameBuffer to render our scene into.
      int width  = 400;
      int height = 600;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(left, right, bottom, top, 8, 12);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);
      // Push the axes away from where the camera is.
      axes_p.setMatrix( Matrix.translate(0, 0, -near) );

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      ModelShading.setColor(modelP, Color.blue);
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);

      // Translate along the x-axis while rotating around the y-axis.
      for (int i = 0; i <= 100; ++i)
      {
         // Push the model P away from where the camera is.
         modelP_p.setMatrix( Matrix.translate(0, 0, -near) );

         // Translate down the y-axis and then rotate around the x-axis.
         // The order of these transformations does matter.
         modelP_p.getMatrix().mult( Matrix.translate(0, i*0.05, 0) );
         modelP_p.getMatrix().mult( Matrix.rotateX(i*18) );

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_24_Frame%03d.ppm", i));
      }
   }
}
