/*

*/

import  renderer.scene.*;
import renderer.scene.primitives.*;

import java.awt.Color;

/**
   Create a wireframe model of a decorated square in the xy-plane
   with it lower left-hand corner at the origin. The decorations
   are meant to break up the square's symmetry, so that we can
   distinguish each corner, no matter what is square's orientation.
<p>
   Here is a picture showing how the decorated square's vertices
   are labeled.
<pre>{@code
        y
        |
        |
        |          v5
     v1 +----------+-----------+ v2
        |                      |
        |                      |
        |                      |
        |                      |
     v4 +          +           + v6
        |          v8          |
        |                      |
        |                      |
        |                      |
        +----------+-----------+------> x
        v0         v7          v3

}</pre>
*/
public class Square extends Model
{
   /**
      Create a square in the xy-plane with corners {@code (1, 1, 0)}.

      @param name  a {link String} that is a name for this {@link Model}
   */
   public Square(final String name)
   {
      this(name, 1);
   }


   /**
      Create a square in the xy-plane with its lower left-hand corner at
      {@code (0, 0, 0)} and its upper right-hand corner at{@code (r, r, 0)}.

      @param name  a {link String} that is a name for this {@link Model}
      @param r     determines the corners of the square
   */
   public Square(final String name, double r)
   {
      super(name);

      r = Math.abs(r);

      // Create the square's perimeter.
      addVertex(new Vertex(0, 0, 0),
                new Vertex(0, r, 0),
                new Vertex(r, r, 0),
                new Vertex(r, 0, 0));

      addPrimitive(new LineSegment(0, 1),
                   new LineSegment(1, 2),
                   new LineSegment(2, 3),
                   new LineSegment(3, 0));

      addColor(Color.blue,
               Color.yellow,
               Color.magenta,
               Color.green);

      // Create the square's "decorations".
      addVertex(new Vertex( 0,  r/2, 0),  // center left edge
                new Vertex(r/2,  r,  0),  // center top edge
                new Vertex( r,  r/2, 0),  // center right edge
                new Vertex(r/2,  0,  0),  // center bottom edge
                new Vertex(r/2, r/2, 0),  // center of square
                new Vertex(3*r/4, r/4, 0)); // lower right quadrant

      addPrimitive(new LineSegment(4, 7),
                   new LineSegment(8, 5),
                   new LineSegment(8, 6),
                   new LineSegment(3, 9));

      addColor(new Color(0.5f, 0.5f, 0.5f),
               new Color(1.0f, 0.5f, 0.5f),
               new Color(0.5f, 0.5f, 0.5f),
               new Color(0.0f, 0.5f, 0.5f),
               Color.red,
               Color.green);
   }
}//Square
