/*
 * Renderer 8. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.util.Assets;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import java.io.File;

/**
   Compare with
      http://threejs.org/examples/#webgl_geometries
   or
      https://stemkoski.github.io/Three.js/Shapes.html
   or
      http://www.smartjava.org/ltjs/chapter-02/04-geometries.html
*/
public class Geometries_R8
{
   private static final String assets = Assets.getPath();

   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Create a two-dimensional array of Positions holding Models.
      final Position[][] position = new Position[5][3];

      // row 0 (first row in first image)
      position[0][0] = new Position(new TriangularPrism(1.0, 1.0, 10));
      ModelShading.setColor(position[0][0].getModel(),
                            Color.green.darker().darker());

      position[0][1] = new Position(new Cylinder(0.5, 1.0, 30, 30));
      ModelShading.setColor(position[0][1].getModel(),
                            Color.blue.brighter().brighter());

      position[0][2] = new Position(new ObjSimpleModel(new File(
                          assets + "great_rhombicosidodecahedron.obj")));
      ModelShading.setColor(position[0][2].getModel(), Color.red);

      // row 1
      position[1][0] = new Position(new GRSModel(new File(
                          assets + "grs/bronto.grs")));
      ModelShading.setColor(position[1][0].getModel(), Color.red);

      position[1][1] = new Position(new ObjSimpleModel(new File(
                          assets + "horse.obj")));
      ModelShading.setColor(position[1][1].getModel(), Color.pink.darker());

      position[1][2] = new Position(new ConeFrustum(0.5, 1.0, 1.0, 10, 10));
      ModelShading.setColor(position[1][2].getModel(), Color.orange.darker());

      // row 2
      position[2][0] = new Position(new Torus(0.75, 0.25, 30, 30));
      ModelShading.setColor(position[2][0].getModel(), Color.gray);

      position[2][1] = new Position(new Octahedron(6));
      ModelShading.setColor(position[2][1].getModel(), Color.green);

      position[2][2] = new Position(new Box(1.0, 1.0, 1.0));
      ModelShading.setRandomPrimitiveColors(position[2][2].getModel());

      // row 3
      position[3][0] = new Position(new ParametricCurve(
                t -> 0.3*(Math.sin(t) + 2*Math.sin(2*t)) + 0.1*Math.sin(t/6),
                t -> 0.3*(Math.cos(t) - 2*Math.cos(2*t)) + 0.1*Math.sin(t/6),
                t -> 0.3*(-Math.sin(3*t)),
                0, 6*Math.PI, 120));
      ModelShading.setRandomPrimitiveColors(position[3][0].getModel());

      position[3][1] = new Position(new ObjSimpleModel(new File(
                          assets + "small_rhombicosidodecahedron.obj")));
      ModelShading.setColor(position[3][1].getModel(), Color.magenta);

      position[3][2] = new Position(new SurfaceOfRevolution(
                t -> 1.5*(0.5 + 0.15 * Math.sin(10*t+1.0)*Math.sin(5*t+0.5)),
                -0.1, 0.9,
                30, 30));
      ModelShading.setColor(position[3][2].getModel(), Color.blue);

      // row 4 (last row in first image)
      position[4][0] = new Position(new Cone(0.5, 1.0, 30, 30));
      ModelShading.setColor(position[4][0].getModel(), Color.yellow);

      position[4][1] = new Position(new Tetrahedron(12, 12));
      ModelShading.setColor(position[4][1].getModel(),
                            Color.green.brighter().brighter());

      position[4][2] = new Position(new Sphere(1.0, 30, 30));
      ModelShading.setColor(position[4][2].getModel(),
                            Color.cyan.brighter().brighter());

      // Create x, y and z axes
      final Position xyzAxes = new Position(
                                  new Axes3D(6, -6, 6, 0, 7, -7, Color.red));

      // Create a horizontal coordinate plane model.
      final Position xzPlane = new Position(new PanelXZ(-6, 6, -7, 7));
      ModelShading.setColor(xzPlane.getModel(), Color.darkGray);

      // Add the positions (and their models) to the Scene.
      scene.addPosition(xzPlane); // draw the grid first
      scene.addPosition(xyzAxes); // draw the axes on top of the grid
      for (int i = position.length - 1;  i >= 0; --i) // from back to front
      {
         for (int j = 0; j < position[i].length; ++j)
         {
            scene.addPosition(position[i][j]);
         }
      }

      // Set up the camera's view frustum.
      final double right  = 2.0;
      final double left   = -right;
      final double top    = 1.0;
      final double bottom = -top;
      final double near   = 1.0;
      scene.getCamera().projPerspective(left, right, bottom, top, near);
/*
      final double fov    = 90.0;
      final double aspect = 2.0;
      final double near   = 1.0;
      scene.getCamera().projPerspective(fov, aspect, near);
*/


      // Create a framebuffer to render our scene into.
      final int vp_width  = 1800;
      final int vp_height =  900;
      final FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      final long startTime, stopTime;
      startTime = System.currentTimeMillis();
      for (int k = 0; k < 360; ++k)
      {
         // Place the xz-plane model in front of the camera
         // and rotate the plane.
         xzPlane.matrix2Identity()
                .mult( Matrix.translate(0, -3, -10) )
                .mult( Matrix.rotateY(k) );

         // Place the xyz-axes model in front of the camera
         // and rotate the axes.
         xyzAxes.matrix2Identity()
                .mult( Matrix.translate(0, -3, -10) )
                .mult( Matrix.rotateY(k) );

         // Place each model where it belongs in the rotated xz-plane
         // and also rotate each model on its own axis.
         for (int i = 0; i < position.length; ++i)
         {
            for (int j = 0; j < position[i].length; ++j)
            {
               // Push the model away from the camera.
               // Rotate the plane of the models.
               // Place the model where it belongs in the rotated plane.
               // Then rotate the model on its own axis.
               position[i][j].matrix2Identity()
                             .mult( Matrix.translate(0, -3, -10) )
                             .mult( Matrix.rotateY(k) )
                             .mult( Matrix.translate(-4+4*j, 0, 6-3*i) )
                             .mult( Matrix.rotateX(3*k) )
                             .mult( Matrix.rotateY(3*k) );
            }
         }

         // Render
         Rasterize.doAntiAliasing = true;
         Rasterize.doGamma = true;
       //fb.clearFB(Color.darkGray.darker());
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Geometries_R8_Frame%03d.ppm", k));
       //fb.dumpFB2File(String.format("PNG_Geometries_R8_Frame%03d.png", k), "png");
      }
      stopTime = System.currentTimeMillis();
      System.out.println("Wall-clock time: " + (stopTime - startTime));
   }
}
