/*
 * Renderer 8. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

package renderer.pipeline;

import renderer.scene.*;
import renderer.scene.util.CheckModel;
import renderer.framebuffer.*;
import static renderer.pipeline.PipelineLogger.*;

import java.awt.Color;

/**
   This renderer takes as its input a {@link Scene} data structure
   and a {@link FrameBuffer.Viewport} within a {@link FrameBuffer}
   data structure. This renderer mutates the {@link FrameBuffer.Viewport}
   so that it is filled in with the rendered image of the geometric
   scene represented by the {@link Scene} object.
<p>
   This implements our seventh rendering pipeline. It converts the
   transformation stage {@link Model2View} to use {@link Matrix}
   transformations instead of just using {@link Vector} translations.
   There are still six pipeline stages.
*/
public final class Pipeline2
{
   // Mostly for compatibility with renderers 1 through 3.
   public static Color DEFAULT_COLOR = Color.white;

   // Make all the intermediate Scene objects
   // available for special effects processing.
   public static Scene scene1 = null;
   public static Scene scene2 = null;
   public static Scene scene3 = null;
   public static Scene scene4 = null;
   public static Scene scene5 = null;

   /**
      Mutate the {@link FrameBuffer}'s default {@link FrameBuffer.Viewport}
      so that it holds the rendered image of the {@link Scene} object.

      @param scene  {@link Scene} object to render
      @param fb     {@link FrameBuffer} to hold rendered image of the {@link Scene}
   */
   public static void render(final Scene scene, final FrameBuffer fb)
   {
      render(scene, fb.vp); // render into the default viewport
   }


   /**
      Mutate the {@link FrameBuffer}'s given {@link FrameBuffer.Viewport}
      so that it holds the rendered image of the {@link Scene} object.

      @param scene  {@link Scene} object to render
      @param vp     {@link FrameBuffer.Viewport} to hold rendered image of the {@link Scene}
   */
   public static void render(final Scene scene, final FrameBuffer.Viewport vp)
   {
      logMessage("\n== Begin Rendering of Scene (Pipeline 2): " + scene.name + " ==");

      scene1 = new Scene(scene.getCamera());

      logMessage("==== 1. Begin model-to-view transformation of Scene ====");
      for (final Position position : scene.positionList)
      {
         if ( position.visible )
         {
            logMessage("==== 1. Render position: "
                             + position.getName() + " ====");

            logMessage("---- Transformation matrix:\n" + position.getMatrix());

            if ( position.getModel().visible )
            {
               logMessage("==== 1. Model-to-view transform of: "
                                + position.getModel().getName() + " ====");

               CheckModel.check(position.getModel());

               // Mostly for compatibility with renderers 1 through 3.
               if (  position.getModel().colorList.isEmpty()
                 && !position.getModel().vertexList.isEmpty())
               {
                  for (int i = 0; i < position.getModel().vertexList.size(); ++i)
                  {
                     position.getModel().addColor( DEFAULT_COLOR );
                  }
                  System.err.println("***WARNING: Added default color to model: "
                                    + position.getModel().getName() + ".");
               }

               logVertexList("0. Model    ", position.getModel());

               final Model model2 = Model2View.model2view(position);

               logVertexList("1. View     ", model2);

               scene1.addPosition( new Position(model2) );

               logMessage("==== 1. End Model: "
                                + model2.getName() + " ====");
            }
            else
            {
               logMessage("==== 1. Hidden model: "
                                + position.getModel().getName() + " ====");
            }

            logMessage("==== 1. End position: "
                             + position.getName() + " ====");
         }
         else
         {
            logMessage("==== 1. Hidden position: "
                             + position.getName() + " ====");
         }
      }
      logMessage("==== 1. End model-to-view transformation of Scene ====");

      scene2 = new Scene(scene1.getCamera());

      logMessage("==== 2. Begin view-to-camera transformation of Scene ====");
      for (final Position position : scene1.positionList)
      {
         logMessage("==== 2. Transform model: "
                          + position.getModel().getName() + " ====");

         final Model model2 = View2Camera.view2camera(position.getModel(),
                                                      scene.getCamera());

         logVertexList("2. Camera   ", model2);
         logColorList("2. Camera   ", model2);
         logPrimitiveList("2. Camera   ", model2);

         scene2.addPosition( new Position(model2) );

         logMessage("==== 2. End Model: " + model2.getName() + " ====");
      }
      logMessage("==== 2. End view-to-camera transformation of Scene ====");

      scene3 = new Scene(scene2.getCamera());

      logMessage("==== 3. Begin near-plane clipping of Scene ====");
      for (final Position position : scene2.positionList)
      {
         logMessage("==== 3. Near_Clip model: "
                          + position.getModel().getName() + " ====");

         final Model model2 = NearClip.clip(position.getModel(),
                                            scene.getCamera());

         logVertexList("3.  Near_Clipped  ", model2);
         logColorList("3.  Near_Clipped  ", model2);
         logPrimitiveList("3.  Near_Clipped  ", model2);

         scene3.addPosition( new Position(model2) );

         logMessage("==== 3. End Model: " + model2.getName() + " ====");
      }
      logMessage("==== 3. End near-plane clipping of Scene ====");

      scene4 = new Scene(scene3.getCamera());

      logMessage("==== 4. Begin projection transformation of Scene ====");
      for (final Position position : scene3.positionList)
      {
         logMessage("==== 4. Project model: "
                          + position.getModel().getName() + " ====");

         final Model model2 = Projection.project(position.getModel(),
                                                 scene.getCamera());

          logVertexList("4. Projected", model2);
        //logPrimitiveList("4. Projected  ", model2);

         scene4.addPosition( new Position(model2) );

          logMessage("==== 4. End Model: " + model2.getName() + " ====");
      }
      logMessage("==== 4. End projection transformation of Scene ====");

      scene5 = new Scene(scene4.getCamera());

      logMessage("==== 5. Begin clipping of Scene ====");
      for (final Position position : scene4.positionList)
      {
         logMessage("==== 5. Clip model: "
                          + position.getModel().getName() + " ====");

         final Model model2 = Clip.clip(position.getModel());

         logVertexList("5. Clipped  ", model2);
         logColorList("5. Clipped  ", model2);
         logPrimitiveList("5. Clipped  ", model2);

         scene5.addPosition( new Position(model2) );

         logMessage("==== 5. End Model: " + model2.getName() + " ====");
      }
      logMessage("==== 5. End clipping of Scene ====");

      logMessage("==== 6. Begin rasterization of Scene ====");
      for (final Position position : scene5.positionList)
      {
         logMessage("==== 6. Rasterize model: "
                          + position.getModel().getName() + " ====");

          Rasterize.rasterize(position.getModel(), vp);

         logMessage("==== 6. End Model: "
                          + position.getModel().getName() + " ====");
      }
      logMessage("==== 6. End rasterization of Scene ====");

      logMessage("== End Rendering of Scene (Pipeline 2) ==");
   }



   // Private default constructor to enforce noninstantiable class.
   // See Item 4 in "Effective Java", 3rd Ed, Joshua Bloch.
   private Pipeline2() {
      throw new AssertionError();
   }
}
