/*
 * Renderer 8. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

package renderer.scene;

/**
   A {@code Position} data structure represents a geometric object
   in a distinct location in three-dimensional view space as part
   of a {@link Scene}.
<p>
   A {@code Position} object holds references to a {@link Model} object
   and a {@link Matrix} object. The {@link Model} represents the geometric
   object in the {@link Scene}. The {@link Matrix} determines the model's
   location and orientation in the {@link Camera}'s view coordinate system.
   The {@code Position}'s matrix helps us solve the problem of placing
   and moving a model in a scene.
<p>
   When the renderer renders this {@code Position}'s {@link Model} into
   a {@link renderer.framebuffer.FrameBuffer}, the first stage of the
   rendering pipeline, {@link renderer.pipeline.Model2View}, multiplies
   every {@link Vertex} in the {@link Model}'s vertex list by this
   {@code Position}'s {@link Matrix}, which converts the coordinates in
   each {@link Vertex} from the model's own local coordinate system to
   the {@link Camera}'s view coordinate system (which is "shared" by all
   the other models in the scene). This matrix multiplication has the effect
   of "placing" the model in view space at an appropriate location (using
   the translation part of the matrix) and in the appropriate orientation
   (using the rotation part of the matrix).
*/
public final class Position
{
   private Model model;
   private Matrix matrix;
   public final String name;
   public boolean visible;
   public boolean debug;

   /**
      Construct a {@code Position} with the identity {@link Matrix}
      and the given {@link Model} object.

      @param model  {@link Model} object to place at this {@code Position}
      @throws NullPointerException if {@code model} is {@code null}
   */
   public Position(final Model model)
   {
      this(model,
           model.getName(),     // default Position name
           Matrix.identity(),   // default matrix
           true,                // visible
           false);              // debug
   }


   /**
      Construct a {@code Position} with the identity {@link Matrix},
      the given name, and the given {@link Model} object.

      @param model  {@link Model} object to place at this {@code Position}
      @param name   {@link String} name for this {@code Position}
      @throws NullPointerException if {@code model} is {@code null}
      @throws NullPointerException if {@code name} is {@code null}
   */
   public Position(final Model model, final String name)
   {
      this(model,
           name,
           Matrix.identity(),   // default matrix
           true,                // visible
           false);              // debug
   }


   /**
      Construct a {@code Position} object with all the given data.

      @param model    {@link Model} object to place at this {@code Position}
      @param name     {@link String} name for this {@code Position}
      @param matrix   transformation {@link Matrix} for this {@code Position}
      @param visible  boolean that determines this {@code Position}'s visibility
      @param debug    boolean that determines if this {@code Position} is logged
      @throws NullPointerException if {@code model} is {@code null}
      @throws NullPointerException if {@code matrix} is {@code null}
      @throws NullPointerException if {@code name} is {@code null}
   */
   public Position(final Model model,
                   final String name,
                   final Matrix matrix,
                   final boolean visible,
                   final boolean debug)
   {
      if (null == model)
         throw new NullPointerException("model must not be null");
      if (null == name)
         throw new NullPointerException("name must not be null");
      if (null == matrix)
         throw new NullPointerException("matrix must not be null");

      this.model = model;
      this.matrix = matrix;
      this.name = name;
      this.visible = visible;
      this.debug = debug;
   }


   /**
      Get a reference to this {@code Position}'s name.

      @return a reference to this {@code Position}'s name
   */
   public String getName()
   {
      return this.name;
   }


   /**
      Get a reference to this {@code Position}'s {@link Model} object.

      @return a reference to this {@code Position}'s {@link Model} object
   */
   public Model getModel()
   {
      return this.model;
   }


   /**
      Set this {@code Position}'s {@link Model} object.

      @param model  {@link Model} object to place at this {@code Position}
      @return a reference to the new {@link Model} object to facilitate chaining method calls
      @throws NullPointerException if {@code model} is {@code null}
   */
   public Model setModel(final Model model)
   {
      if (null == model)
         throw new NullPointerException("model must not be null");

      this.model = model;
      return this.model;
   }


   /**
      Get a reference to this {@code Position}'s {@link Matrix} object.

      @return a reference to this {@code Position}'s {@link Matrix} object
   */
   public Matrix getMatrix()
   {
      return this.matrix;
   }


   /**
      Set this {@code Position}'s {@link Matrix} object.

      @param matrix  {@link Matrix} object for this {@code Position}
      @return a reference to the new {@link Matrix} object to facilitate chaining method calls
      @throws NullPointerException if {@code matrix} is {@code null}
   */
   public Matrix setMatrix(final Matrix matrix)
   {
      if (null == matrix)
         throw new NullPointerException("matrix must not be null");

      this.matrix = matrix;
      return this.matrix;
   }


   /**
      Reset this {@code Position}'s {@link Matrix} to the identity matrix.

      @return a reference to this {@code Position}'s {@link Matrix} to facilitate chaining method calls
   */
   public Matrix matrix2Identity()
   {
      this.matrix = Matrix.identity();
      return this.matrix;
   }


   /**
      Set this {@code Position}'s {@link Matrix} object
      to a translation {@link Matrix}.

      @deprecated  Use the {@link Matrix} API instead of this method.
         This method is here for compatability with renderers 1 through 7.

      @param dx  translation amount in the x-direction
      @param dy  translation amount in the y-direction
      @param dz  translation amount in the z-direction
   */
   @Deprecated
   public void translation(final double dx,
                           final double dy,
                           final double dz)
   {
      this.matrix = Matrix.translate(dx, dy, dz);
   }


   /**
      For debugging.

      @return {@link String} representation of this {@code Position} object
   */
   @Override
   public String toString()
   {
      String result = "";
      result += "Position: " + name + "\n";
      result += "This Position's visibility is: " + visible + "\n";
      result += "This Position's Matrix is\n";
      result += matrix + "\n";
      result += "This Position's Model is\n";
      result += (null == model) ? "null\n" : model;
      return result;
   }
}
